@@ -1,5 +1,7 @@
 #!/bin/bash
 
+set -e
+
 CONDA_INSTALL="conda install -q -y"
 PIP_INSTALL="pip install -q"
 
@@ -25,7 +27,11 @@ source activate $CONDA_ENV
 set -v
 
 # Install llvmdev (separate channel, for now)
-$CONDA_INSTALL -c numba llvmdev="6.0*"
+if [ "$LLVM" == "15" ]; then
+    $CONDA_INSTALL -c conda-forge llvmdev="15"
+else
+    $CONDA_INSTALL -c numba/label/dev llvmdev="14.*"
+fi
 
 # Install the compiler toolchain, for osx, bootstrapping needed
 # which happens in build.sh
@@ -33,18 +39,5 @@ if [[ $(uname) == Linux ]]; then
 $CONDA_INSTALL gcc_linux-64 gxx_linux-64
 fi
 
-# Install enum34 for Python < 3.4 and PyPy, and install dependencies for
-# building the docs. Sphinx 1.5.4 has a bug.
-if [ "$PYTHON" == "pypy" ]; then
-  python -m ensurepip
-  $PIP_INSTALL enum34
-  $PIP_INSTALL sphinx==1.5.1 sphinx_rtd_theme pygments
-else
-  $CONDA_INSTALL sphinx=1.5.1 sphinx_rtd_theme pygments
-  if [ "$PYTHON" \< "3.4" && "$WHEEL" != "yes" ]; then
-    $CONDA_INSTALL enum34
-  fi
-fi
-
 # Install dependencies for code coverage (codecov.io)
 if [ "$RUN_COVERAGE" == "yes" ]; then $PIP_INSTALL codecov coveralls; fi
